# Produces list of tWs for each verse by linking MAST_NT to ULB through tWs.
# Disambiguates entries found on more than one tW page.
# Does both Testaments.

# Taken from tWs.from.MAST_NT.2.pl.

# The output from this script is useful for the interleaved PDFs used in MAST.
# This version uses an exception file to handle places where the MAST_NT points to
# a tW page different from that on which the ULB term appears.

# Make sure the correct input file is $ULBfile. Run script.
# Output is in $output file.
# Check the $missing
# file for needed corrections, probably lines needing to be added to the
# $exceptions file.

use 5.12.0;
use File::Slurp;
use File::Find ; 
use Cwd ;
use FindBin '$Bin';
use utf8;
#use open IN => ":utf8", OUT => ":utf8";
use open IO => ":utf8";
use List::MoreUtils qw(uniq);
$|=1;
$"="\n";

my ($pwd, $d) = ($Bin, "/");
if ($^O eq "MSWin32") {
	$d = "\\";
	$pwd =~ s/\//\\/g;
}

my ($udf) = "User_defaults.windows.txt";
if ($^O eq "linux") {$udf = "User_defaults.linux.txt"}
#elsif ($^O eq "darwin") {$udf = "User_defaults.mac.txt"}
elsif ($^O eq "darwin") {$udf = "User_defaults.mac.txt"}

open (my $defaults, "<:utf8", "$pwd${d}User${d}$udf") or die "$pwd${d}User${d}$udf:\n$!";

my ($cv, $ULBfile, $exceptions, $missing, $output, $workFile) =
					("00000",
					"$Bin${d}Temp${d}Extract.txt",
					"$Bin${d}Exceptions${d}Exceptions.txt",
					"$Bin${d}Output${d}Entries_not_handled.txt",
					"$Bin${d}Output${d}tWs_for_PDF.txt",
					"$Bin${d}User${d}tW_work.txt",
					);

my ($sourceFile, $topTwDir, $textEditor, $repoPath, $topSourceLangDir, $topOTSourceLangDir, $topNTSourceLangDir, $twPath, $finalOutString);

my (%entries, %ULBtext, %orderRef, %order, %pages, %listOfPages, %sourcePage, %checkPages, %foundPages, %substitutedPages,
	 %specifiedText, %realPage, %checkPage, %workEntries, %vsn, %StrongNum, %fullText, %ulbOrder, %bkAbr, %bkFull, %relevantSNs,
	 %relevantSNsInCV, %entriesThisSN, %entriesThisPage, %pagesThisSN, %pagesThisEntry, %adjust, %deleteNum, %newRef,
	 %specifiedEntries, %finalOutput, %TWDir, %tempRef, %dir, %rawDir);

my ($book, $testament);
my (@fileList);

# ==============================

chdir("$pwd");
open LOG, ">:utf8", "Logs${d}1_Data_and_inputs.txt" or die "\$log: Logs${d}1_Data_andinputs.txt: $!";
open OUT, ">:utf8", $output or die "$!";
open MISSING, ">$missing" or die "$!";

while (<DATA>) {
	chomp;
	if (/([^\t]*)\t([^\t]*)/) {
		$bkAbr{$2} = $1;
		$bkFull{$1} = $2;
	} elsif (/^..$/) {
		$testament = $&;
	}
}

GetUserDefaults();
GetULBBooksToProcess();
ReadExceptions();
close LOG;
open LOG, ">:utf8", "Logs${d}2_tW_pairs_log.txt" or die "Logs${d}2_tW_pairs_log.txt: $!";
PairtWEntriesTotWPageAndUniqSNs();
close LOG;
open LOG, ">:utf8", "Logs${d}3_tWs_from_MAST_log.txt" or die "3_tWs_from_MAST_log.txt: $!";
GetRelevantSNsForEachVerse();
LinkULBtoCV();
close LOG;
open LOG, ">:utf8", "Logs${d}4_Process_log.txt" or die "4_Process_log.txt: $!";
ProcessEachVerse();

say OUT $finalOutString;

close MISSING;
close OUT;
close LOG;

if ($^O eq "darwin") {system ("$textEditor $missing")}

print "\n\tDone.\n\n";

# ==============================

sub GetUserDefaults {
	open (my $defaults, "<:utf8", "User${d}$udf") or die "User${d}$udf:\n$!";
	
		while (my $thisLine = <$defaults>) {
			chomp $thisLine;
			if ($thisLine =~ /^Text editor: (.*)$/) {
				$textEditor = $1;
				if ($^O eq "darwin") {
					$textEditor = "open -a $textEditor"
				}
			} elsif ($thisLine =~ /^Repository directory: (.*)$/) {
				$repoPath = $1;
				#say $repoPath; die;
			} elsif ($thisLine =~ /^translationWords path: (.*)$/) {
				$twPath = $1;
				#say $repoPath; die;
			}
		}

		#say LOG "\$textEditor: $textEditor\n\$repoPath: $repoPath";
		die "No text editor found" if $textEditor eq "";
		die "No path to repo found" if $repoPath eq "";
		
		($topTwDir, $topOTSourceLangDir, $topNTSourceLangDir) = ("$repoPath${d}$twPath", "$repoPath${d}MAST_HB", "$repoPath${d}OGNT");
		
		say LOG "\$topTwDir: $topTwDir\n\$topOTSourceLangDir: $topOTSourceLangDir\n\$topNTSourceLangDir: $topNTSourceLangDir	";
	
	close $defaults;
}

sub GetULBBooksToProcess {
	say LOG "GetULBBooksToProcess on \$workFile: $workFile";
	open (my $file, "<:utf8", "$workFile") or die "$workFile:\n$!";
	
		while (my $line = <$file>) {
			chomp $line;
			#say LOG "\t$line";
			if ($line =~ /^([^#][^\n\t]*)\t[^\n\t]*\t([^\n\t]*)\t[^\n\t]*$/) {
				my ($bn, $bx) = ($1, $2);
				my ($this_bk) = $bn . "-" . uc $bx;
				if ($bn > 39) {
					$topSourceLangDir = $topNTSourceLangDir
				} else {
					$topSourceLangDir = $topOTSourceLangDir
				}
				
				$sourceFile = "$topSourceLangDir${d}$this_bk.xml";
				say LOG $sourceFile;
				push @fileList, $sourceFile;
			}
		}
	
	close $file;
	say LOG "===\n\@fileList:\n@fileList\n===\n";
}

sub ReadExceptions {
	say "Reading exceptions";
	say LOG "ReadExceptions from \$exceptions: $exceptions";
	open (my $file, "<:utf8", "$exceptions") or die "$exceptions:\n$!";
	#say LOG "<><>";
	while (my $line = <$file>) {
		chomp $line;
		#say LOG $line;
		my $rf;
		if ($line =~ /^([^#\n][^\t\n]*)\t([GH]\d+)\t\|\|$/) {
			my ($SNtoSkip) = ($2);
			$rf = $1;
			say LOG "<1>\t\$line: $line, \$rf: $rf, \$SNtoSkip: $SNtoSkip";
			($deleteNum{$rf}) .= "$SNtoSkip√";
			$specifiedText{$rf} = 1;
			#say LOG "\$specifiedText{$rf}: $specifiedText{$rf}";
		} elsif ($line =~ /^([^#\n][^\t\n]*)\t([GH]\d+\t[GH]\d+)/) {
			my ($oldNew) = ($2);
			$rf = $1;
			say LOG "<2>\t\$line: $line, \$rf: $rf, \$oldNew: $oldNew";
			($adjust{$rf}) .= "$oldNew√";
			$specifiedText{$rf} = 1;
		}
		elsif ($line =~ /^([^#\n\t][^\t\n]*)\t(.\d+)\t([^\t\n]*)\t([^\t\n]*)$/) {
			my ($rf, $sn, $snippet, $page) = ($1, $2, $3, $4);
			say LOG "<3>\t\$rf: $rf,	 \$sn: $sn,	 \$snippet: $snippet,	 \$page: $page	";
			$specifiedEntries{$rf} .= "$sn≈$snippet≈$page√";
			$relevantSNsInCV{$rf} =~ s/$sn√?//;
			$specifiedText{$rf} = 1;
		}

	}
	close $file;
}

sub PairtWEntriesTotWPageAndUniqSNs {
	say "Pairing tW entries with tW pages and unique Strong's numbers in $topTwDir";
	my (@filesToRun, @relevantSNs) = ();
	my $filePattern = '*.md' ; 
	find (sub {push @filesToRun, $File::Find::name if (m/^(.*)$filePattern$/)}, $topTwDir) ;
	@filesToRun = sort @filesToRun;
	say LOG "\@filesToRun: @filesToRun";
	foreach my $file (@filesToRun) {
		print ".";
		$file =~ s/\//\\/g unless $^O eq "linux" || $^O eq "darwin"; 
		my ($thisList, $shortFile) = ("", $file);
		print LOG "\$file: $file\n\$shortFile: $shortFile";
		$shortFile =~ s/^\Q$topTwDir${d}\E//;
		print LOG " =1> $shortFile";
		$shortFile =~ s/\.md$//;
		print LOG " =2> $shortFile";
		$shortFile =~ s/\Q$d\E/,/;
		if ($file =~ /\/([^\/]*)\/([^\/]*)\.md/) {
			$dir{$shortFile} = $1;
			$rawDir{$2} = $1;
		}
		say LOG " =3> \$shortFile: $shortFile\t\$dir{$shortFile}: $dir{$shortFile}";
		#say "|$shortFile|"; die;
		#if ($shortFile =~ /^(kt|names)/) {
		#my $fileText = read_file("$file", binmode => 'utf8');
		my $fileText = read_file("$file", binmode => 'utf8');
		$fileText =~ s/$/\n/;
		#say LOG "=====\n$file:\n$fileText";
		if ($fileText =~ /## Forms Found in the English ULB:\n\n([^\n]*)\n/s) {
			$thisList = $1;
			#say LOG "$file: $thisList";
			$thisList =~ s/[\r\n]+$//;
			my @ULBEntries = split /, /, $thisList;
			my @tempArray = split /, /, $thisList;
			foreach my $ULB_entry(@ULBEntries) {
				if (exists $pagesThisEntry{$ULB_entry}) {
					say LOG "*!!*"
				}
				$pagesThisEntry{$ULB_entry} .= "$shortFile, ";
				$sourcePage{$ULB_entry} = $shortFile;
				say LOG "\$pagesThisEntry{$ULB_entry}: $pagesThisEntry{$ULB_entry}, \$sourcePage{$ULB_entry}: $sourcePage{$ULB_entry}, \$shortFile: $shortFile"
			}
		}
		if ($fileText =~ /Strong's([^\n]*)\n/) {
			my $SNs = $1;
			while ($SNs =~ s/([GH]\d*)//) {
				my $thisSN = $1;
				print LOG "! $shortFile ! $thisSN !";
				push @relevantSNs, $thisSN;
				$entriesThisSN{$thisSN} .= "$thisList, ";
				$pagesThisSN{$thisSN} .= "$shortFile, ";
				say LOG " \$pagesThisSN{$thisSN}: $pagesThisSN{$thisSN}"
			}
		}


	}
	@relevantSNs = uniq(@relevantSNs);
	foreach (@relevantSNs) {
		$relevantSNs{$_} = "$_";
	}
	say "";
	#say LOG "====";
	#say LOG "====";
	foreach my $thisSN (sort keys %entriesThisSN) {
		$entriesThisSN{$thisSN} =~ s/, $//;
		#say LOG "\$entriesThisSN{$thisSN}: $entriesThisSN{$thisSN}, \$pagesThisSN{$thisSN}: $pagesThisSN{$thisSN}"
	}
	foreach my $entry (sort keys %pagesThisEntry) {
		print LOG "\$entry: $entry, \$pagesThisEntry{$entry}: $pagesThisEntry{$entry}, ";
		$pagesThisEntry{$entry} =~ s/, $//;
		say LOG "=> $pagesThisEntry{$entry}"
	}
	foreach my $sn (sort keys %pagesThisSN) {
		print LOG "\$sn: $sn, \$pagesThisSN{$sn}: |$pagesThisSN{$sn}|\t";
		$pagesThisSN{$sn} =~ s/, $//;
		say LOG "=>\t|$pagesThisSN{$sn}|"
	}
}

sub LinkULBtoCV {
	say "Linking ULB to chapter and verse";
	say LOG "LinkULBtoCV, \$ULBfile: $ULBfile";
	open IN, "$ULBfile" or die "$ULBfile: $!";
		while (<IN>) {
			if (/^([^\t]*)\t(.*)$/) {
				# "\$_:$_";
				$cv ++;
				($ULBtext{$1}, $fullText{$1}) = ($2, $2);
				$orderRef{$cv} = $1;
				$tempRef{$orderRef{$cv}} = $orderRef{$cv};
				$tempRef{$orderRef{$cv}} =~ s/^([^:]*) (\d+):(\d+)/$1,$2,$3/;
				$order{$1} = $cv;
			}
			#say LOG "First \$orderRef{$cv}: $orderRef{$cv}\nGetULBBooksToProcess\n\$ULBtext{$orderRef{$cv}}: $ULBtext{$orderRef{$cv}}";
		}
	close IN;
	foreach my $key (sort keys %orderRef) {
		$ULBtext{$orderRef{$key}} =~ s/[^\w]+$//;
		$ULBtext{$orderRef{$key}} .= " q";
		#say LOG "\$key: $key:, \$orderRef{$key}: $orderRef{$key}, \$ULBtext{$orderRef{$key}}: $ULBtext{$orderRef{$key}}"
	}
}

sub GetRelevantSNsForEachVerse {
	say "Getting relevant SNs for each verse";
	my ($flag, $sourceFile) = ("","");
	foreach $sourceFile (@fileList) {
		my $hg = "G";
		if ($sourceFile =~ /(..)-...\.xml$/) {
			$hg = "H" if ($1 < 40);
		}
		say LOG "opening \$sourceFile: $sourceFile";
		open IN, "$sourceFile" or die "$sourceFile can't be opened\n\n";
			my ($thisBook, $thisChap, $thisVers, $thisCV);
			my (@pages);
			while (<IN>) {
				chomp;
				say LOG "<\@>\t$_";
				if (/<verse osisID="([^\.]*).(\d+).(\d+)">/) {
					#say LOG "$thisCV: \$relevantSNsInCV{$thisCV}: $relevantSNsInCV{$thisCV}";# Making sure previous verse is populated
					my ($bk, $ch, $vs) = ($1, $2, $3);
					#say LOG "#>\t$bk $ch:$vs, \$bkFull{$bk}: $bkFull{$bk}";
					if (exists $newRef{"$bk $ch:$vs"}) {
						$thisCV = $newRef{"$bk $ch:$vs"}
					} else {
						($thisCV) = ("$bkFull{$bk} $ch:$vs");
					}
					#say LOG "##\t$bk $ch:$vs, $thisCV";
				}
				else {
					if (/lemma="([^"]*)"/ && not /<note type="variant">/) {
						my $gist = $1;
						say LOG "<\@\@>\t\$gist: $gist";
						if ($gist =~ /\d+/) {
							s/(lemma=")[^\d]*?(\d+)[^\d]*?(")/$1$2$3/;
							while (/<w lemma="(\d+)"/g) {
								my ($thisSN) = ($hg . $1);
								say LOG "<\@\@\@>\t\$thisSN: $thisSN, \$relevantSNsInCV{$thisCV}: $relevantSNsInCV{$thisCV}";
								if (exists $relevantSNs{$thisSN}) {
									$relevantSNsInCV{$thisCV} .= "$thisSN√" unless ($relevantSNsInCV{$thisCV} =~ /\b$thisSN\b/);
								}
								say LOG ">\t\$thisSN: $thisSN, \$relevantSNsInCV{$thisCV}: $relevantSNsInCV{$thisCV}";
							}
						}
					}
				}
			}
		close IN;
	}
	my %temp;
	foreach my $oldRef (sort keys %relevantSNsInCV) {
		if (exists $newRef{$oldRef}) {
			$temp{$newRef{$oldRef}} = $relevantSNsInCV{$oldRef};
			delete $relevantSNsInCV{$oldRef};
		}
	}
	foreach my $changedRef (sort keys %temp) {
		$relevantSNsInCV{$changedRef} = $temp{$changedRef};
		say LOG "\$relevantSNsInCV{$changedRef}: $relevantSNsInCV{$changedRef}";
	}
}

sub ProcessEachVerse  {
	say "Processing ULB";
	say LOG "sub ProcessEachVerse called";
	foreach my $key (sort keys %orderRef) {
	# for each verse
		my ($thisCV) = ($orderRef{$key});
		say LOG "\n<5>\n$thisCV\t$ULBtext{$thisCV}\n\t\$relevantSNsInCV{$thisCV}: $relevantSNsInCV{$thisCV}\n\t\$deleteNum{$thisCV}: |$deleteNum{$thisCV}|\n\t\$specifiedEntries{$thisCV}: $specifiedEntries{$thisCV}";
		
		($relevantSNsInCV{$thisCV}) = DeleteSpecifiedWords ($relevantSNsInCV{$thisCV}, $specifiedEntries{$thisCV});

		($relevantSNsInCV{$thisCV}) = DeleteObviatedSNs($relevantSNsInCV{$thisCV}, $deleteNum{$thisCV});
		# delete obviated SNs

		say LOG "<6>\t\$relevantSNsInCV{$thisCV}: $relevantSNsInCV{$thisCV}";
		my $processSequence = "$specifiedEntries{$thisCV}√$relevantSNsInCV{$thisCV}";
		$processSequence =~ s/√+/√/g;
		$processSequence =~ s/(^√+|√+$)//g;
		say LOG "\t\$processSequence: $processSequence";

		$finalOutString .= ExecuteProcessSequence($thisCV, $processSequence, $ULBtext{$thisCV});

	}
	
}

sub DeleteSpecifiedWords {
	my ($sns, $toDelete) = @_;
	say LOG "<5.1>\t\$sns:\t\t$sns\n\t\t\$toDelete:\t$toDelete";
	my @sns = split /√/, $sns;
	my @delete = split /√/, $toDelete;
	my %delete;
	foreach my $one (@delete) {
		say LOG "<5.1.1>\t\$one: $one";
		$one =~ s/^([^≈]*)≈.*$/$1/;
		say LOG "<5.1.2>\t\$one: $one";
		$sns =~ s/^(.*)$one(√)?(.*)$/$1$2$3/;
		$delete{$one} = $one;
	}
	my $newSNs;
	foreach my $oldSN (@sns) {
		unless (exists $delete{$oldSN}) {
			$newSNs .= "$oldSN√"
		}
	}
	return $newSNs;
}

sub DeleteObviatedSNs {
	my ($oldSNs, $toDelete) = ($_[0] . "√", $_[1]);
	say LOG "<5.2>\t\$oldSNs:\t$oldSNs\n\t\t\$toDelete:\t$toDelete";
	my @sns = split /√/, $oldSNs;
	my @toDelete = split /√/, $toDelete;
	my %toDelete;
	foreach my $badSN (@toDelete) {
		$toDelete{$badSN} = $badSN;
	}
	my $newSNs = "";
	foreach my $oldSN (@sns) {
		unless (exists $toDelete{$oldSN}) {
			$newSNs .= "$oldSN√"
		}
	}
	return $newSNs
}
sub ExecuteProcessSequence {
	my ($ref, $sequence, $trueText) = @_;
	my ($tempText, $thisCVOutString, $position, $outputFormRef) = ($trueText, "");
	my (%snippetSequence);
	my (@SNsequence) = split /√/, $sequence;
	say LOG "$ref:\n@SNsequence";
	if ($ref =~ /^([^:]*) (\d+):(\d+)/) {
		$outputFormRef = "$1,$2,$3"
	}
	
	foreach my $candidate (@SNsequence) {
		# for each relevant SN in verse
		# 	for each tW entry
		#		if specified tW
		say LOG "=====\n\$ref: $ref\t\$candidate: $candidate\t\$entriesThisSN{$candidate}: $entriesThisSN{$candidate}\n$tempText";
		my ($found, $sn, $ulbWord, $tWpage);
		my( $staticCandidate) = $candidate;
		if ($candidate =~ /([^≈]*)≈([^≈]*)≈([^≈]*)/) {
		#			get position in true text to array
		#			delete found text from temp text
			($sn, $ulbWord, $staticCandidate, $tWpage) = ($1, $2, $2, $3);
			if ($ulbWord =~ s/^(.*) \.\.\. (.*) \.\.\. (.*)/($1)\\b(.*?)\\b($2)\\b(.*?)\\b($3)/) {
				if ($tempText =~ s/^(.*)\b$ulbWord\b(.*)$/$1$3$5$7/) {
					say LOG "<A>\t\$ulbWord: $ulbWord	\$sourcePage{$staticCandidate}: $sourcePage{$staticCandidate}\t\$rawDir{\$tWpage}: $rawDir{$tWpage}	\$dir{\$tWpage}: $dir{$tWpage}";
					say LOG "<B>\t\$1: $1	\$2: $2	\$3: $3	\$4: $4	\$5: $5	\$6: $6	\$7: $7";
					say LOG "<C>\t\$tempText: $tempText";
					$position = length $1;
					$snippetSequence{$position} = "$ulbWord,$sourcePage{$staticCandidate}";
				}
			}
			elsif ($ulbWord =~ s/^(.*) \.\.\. (.*)/($1)\\b(.*?)\\b($2)/) {
				if ($tempText =~ s/^(.*)\b$ulbWord\b(.*)$/$1$3$5/) {
					say LOG "<D>\t\$ulbWord: $ulbWord	\$sourcePage{$staticCandidate}: $sourcePage{$staticCandidate}\t\$rawDir{\$tWpage}: $rawDir{$tWpage}	\$dir{\$tWpage}: $dir{$tWpage}";
					say LOG "<E>\t\$1: $1	\$2: $2	\$3: $3	\$4: $4	\$5: $5	\$6: $6";
					say LOG "<F>\t\$tempText: $tempText";
					$position = length $1;
					$snippetSequence{$position} = "$ulbWord,$sourcePage{$staticCandidate}";
				}
			}
			elsif ($tempText =~ s/^(.*)\b$ulbWord\b(.*)$/$1$2/) {
				say LOG "<G>\t\$ulbWord: $ulbWord	\$sourcePage{$staticCandidate}: $sourcePage{$staticCandidate}\t\$rawDir{\$tWpage}: $rawDir{$tWpage}	\$dir{\$tWpage}: $dir{$tWpage}";
				say LOG "<H>\t\$1: $1	\$2: $2	\$3: $3	\$4: $4	\$5: $5	\$6: $6";
				say LOG "<I>\t\$tempText: $tempText";
				$position = length $1;
				$snippetSequence{$position} = "$ulbWord,$sourcePage{$staticCandidate}";
			}
		}
		#		else
		else {
			my @possibleEntries = split /, /, $entriesThisSN{$candidate};
			@possibleEntries = reverse sort {  substr($a,0,1) <=> substr($b,0,1)
			 || length($a) <=> length($b)
			 || $a <=> $b }
			@possibleEntries;
			foreach my $thisEntry (@possibleEntries) {
				#			for each possible entry;
				my $staticEntry = $thisEntry;
				print LOG "-----\n\$thisEntry: $thisEntry\t=>\t";
				while ($thisEntry =~ s/^(.*) \.\.\. (.*)/$1\\b(.*?)\\b$2/) {}
				say LOG "$thisEntry\t\$pagesThisEntry{$staticEntry}: $pagesThisEntry{$staticEntry}";
				#				if tW entry matches ULB text
				#					get ULB snippet to verse match list
				#					get position in true text to array
				#					delete found text from temp text
				if ($thisEntry =~ /\(\.\*\?\)/ && $tempText =~ /$thisEntry/i) {
					say LOG "<7>\t\$thisEntry |$thisEntry| is found in the first test";
					my ($first, $second, $third, $fourth, $fifth);
					if ($tempText =~ /^(.*)\b($thisEntry)\b(.*)$/i) {
						($first, $second, $third, $fourth, $fifth) = ($!, $2, $3, $4, $5);
						say LOG "<7.1>\t\$first: $first	\$second: $second	\$third: $third	\$fourth: $fourth	\$fifth: $fifth	";
						if ($fifth ne "") {
							$tempText =~ s/^(.*)\b($thisEntry)\b(.*)$/$1$4$5/i
						}
						else {
							$tempText =~ s/^(.*)\b($thisEntry)\b(.*)$/$1$3$4/i
						}
						
					}
					if ($trueText =~ /^(.*)\b($thisEntry)\b.*$/) {
						$position = length $1;
						say LOG "<7.2>\t\$position: $position"
					}
					$snippetSequence{$position} = "$thisEntry,$pagesThisEntry{$staticEntry}";
						$found = 1;
						goto Breakout;
				}
				elsif ($tempText =~ s/\b($thisEntry)[^\w']//i || $tempText =~ s/\b($thisEntry)["']//i || $tempText =~ s/["']($thisEntry)\b//i) {
					say LOG "<7.3>\t\$thisEntry |$thisEntry| is found in the second test\n---
					";
					if ($trueText =~ /^(.*)\b$thisEntry[^\w']/i || $trueText =~ /^(.*)\b($thisEntry)["']/i || $trueText =~ /^(.*)["']($thisEntry)\b/i) {
						$position = length $1;
					}
					$snippetSequence{$position} = "$thisEntry,$pagesThisEntry{$thisEntry}";
						say LOG "<7.4>\t\$snippetSequence{$position}: $snippetSequence{$position}";
						$found = 1;
						goto Breakout;
				}
				else {
						say LOG "\$thisEntry |$thisEntry| is not found in |$ref|";
				}
			}
				Breakout:
				unless ($found) {
					say MISSING "$ref	$candidate";
					say LOG "Breakout: \$ref: $ref\t\$candidate: $candidate"
				}
				next if $found;
		}
		$sequence = $specifiedEntries{$ref};
		$sequence =~ s/[GH]\d+≈([^≈]*)≈([^≈√]*)/$1\t$2/g;
	}
	foreach my $key (sort {$a <=> $b} (keys %snippetSequence)) {
		$snippetSequence{$key} =~ s/\(([^\)]*)\)\\b.*?\\b\(([^\)]*)\)\\b.*?\\b\(([^\)]*)\)/$1 ... $2 ... $3/;
		$snippetSequence{$key} =~ s/\(([^\)]*)\)\\b.*?\\b\(([^\)]*)\)/$1 ... $2/;
		say LOG "---\n\$key: $key\t\$snippetSequence{$key}: $snippetSequence{$key}\n===";
		$snippetSequence{$key} =~ s/(\\b\.\*\?|\.\*\?\\b)//g;
		$snippetSequence{$key} =~ s/\\b\(\.\*\?\)\\b/ ... /g;
		$thisCVOutString .= "$outputFormRef,$snippetSequence{$key}\n"
	}
	say LOG "<7.5>\t\$thisCVOutString\n$thisCVOutString";
	return $thisCVOutString
}

__DATA__
OT
gen	Genesis
exo	ExodusB
lev	Leviticus
num	Numbers
deu	Deuteronomy
jos	Joshua
jdg	Judges
rut	Ruth
1sa	1 Samuel
2sa	2 Samuel
1ki	1 Kings
2ki	2 Kings
1ch	1 Chronicles
2ch	2 Chronicles
ezr	Ezra
neh	Nehemiah
est	Esther
job	Job
psa	Psalms
pro	Proverbs
ecc	Ecclesiastes
sng	Song of Solomon
isa	Isaiah
jer	Jeremiah
lam	Lamentations
ezk	Ezekiel
dan	Daniel
hos	Hosea
jol	Joel
amo	Amos
oba	Obadiah
jon	Jonah
mic	Micah
nam	Nahum
hab	Habakkuk
zep	Zephaniah
hag	Haggai
zec	Zechariah
mal	Malachi
NT
mat	Matthew
mrk	Mark
luk	Luke
jhn	John
act	Acts
rom	Romans
1co	1 Corinthians
2co	2 Corinthians
gal	Galatians
eph	Ephesians
php	Philippians
col	Colossians
1th	1 Thessalonians
2th	2 Thessalonians
1ti	1 Timothy
2ti	2 Timothy
tit	Titus
phm	Philemon
heb	Hebrews
jas	James
1pe	1 Peter
2pe	2 Peter
1jn	1 John
2jn	2 John
3jn	3 John
jud	Jude
rev	Revelation
