# Updating the tW list for PDFs  
  
These procedures work on my Mac. If a version is needed for another machine, I can adapt it, as can anyone familiar with scripts.  
  
## Suggested repository structure

You will find it easiest to use this tool if all of your files are in one place, as in this sample repository, which has been taken directly from the Wycliffe Associates database.

<img align     = "bottom"
     alt       = "sample repo"
     height    = "200pt"
     hspace    = "200"
     src       = "FilesForUpdates/SampleRepoStructure.png"
     />

Here the respective directories are for the translationNotes (en_tn), the translationWords (en_tw, with its three divisions into key terms [kt], names, and other terms), the dynamic translation (en_udb), the literal translation (en_ulb), the Hebrew morphological Bible (OSHB) and the Greek mophological Bible (UGNT).

## Files included in the package

### User files

These files, found in the *User* subdirectory are for you to tell the tool what data to look for, how to look for it, and what to do once it has found it.

The UserDefaults file tells the scripts where you have put the repository (i.e., the ULB, UDB, tN, and tW files) and which text editor to use when it is finished working.

     User.defaults.txt

These other files tell the Perl scripts which books of either the Old Testament or the New Testament you want to work on.

     tW.work.NT.txt
     tW.work.OT.txt

### Main scripts 

These files call the Perl scripts that actually do the work. They are in the form Filename.sh for Mac and Linux, Filename.bat for Windows.

     do.nt.sh
     do.ot.sh
     mine.sh
     update.nt.sh
     update.ot.sh
     waa.sh
  
### Perl scripts  

These are the scripts that do the work for this tool.
  
     CombineULBandNASBwithCodes.pl
     ExtractLinksFromScratchPad.pl
     FindMismatchedULBSnippets.3.noChunks.pl
     GrepBook.NT.pl
     GrepBook.OT.pl
     MakeULB.3.pl
     Mine.URL.Strong.Verse.NT.pl
     Mine.URL.Strong.Verse.OT.pl
     OpenMismatchedFiles.pl
     Output.to.csv.pl
     tWs.from.OSHB.1.pl
     tWs.from.UGNT.8.pl

### Output files

These files are found in the *Output* subdirectory. The file *tWs.for.PDF.txt* is the file you are trying to produce. It is a list of all the translationWords relevant to every verse in the New Testament and the Old Testament. As you are building this file, however, you will find that many of the entries in the source file are not handled. These will be listed in the file *Entries.not.handled.txt*.

### Exceptions files

These files, found in the **Exceptions** subdirectory, tell the Perl scripts how to handle words in each verse that do not fit the tW pages easily. You will be editing these files so that the tool produces the proper **tWs.for.PDF.txt** file.

     Exceptions.tWs.from.OSHB.txt
     Exceptions.tWs.from.UGNT.txt

### Data files

This file in the *Data* subdirectory does not change. It tells the main tool what Strong's codes are in the NASB.

     NASB.Strongs.txt

### Log and temporary files

You should never have to look at these files, which are found in the *Logs* and *Temp* subdirectories. They are produced at different times when the Perl scripts are run and so often change.

#### Log files

     log.log
     mine.log.log

#### Temporary files

     Extract.txt
     UDB text.txt
     ULB text.txt
     ULB.NASB.Strongs.txt
     mismatched_snippets.html

## Procedures

**First run the do shell for the appropriate testament**

    	sh do.ot.sh
    	or
    	sh do.nt.sh
        
This will pull the latest of all files down, update the ULB, and check for missing key words.

**For each missing keyword pair in the missing.log file**

     Select the line, including the end of line
     Run mine.sh

    	sh mine.ot.sh
    	or
    	sh mine.nt.sh
     	
     Paste the copied line in the terminal to answer the prompt
     Choose the best alternative
	* change key word associated with the Strong's number
	* specify ULB text to go with that key word
	* comment out the key word so it is not in the tW list
	
**After dealing with all missing words**

     Push all changes to the Git
     Run do.sh again to make sure no new errors have been introduced
     Run waa.sh to check for and correct ULB–tN snippet mismatches

**After all corrections have been made**

     Run Output.to.csv.pl to update the local repo
     Commit all changes

